/* Statistics Generator C Source */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <conio.h>
#include <ctype.h>
#include <time.h>
#include <io.h>
#include <fcntl.h>

FILE *in, *out, *debugout, *trap;

long traplength;
long numlines = 0;

static char *version = "9.9";
char *configfile = "STATGEN.CFG";

int debug=0;
long seeuser=1;
char usersort='A',
     networksort='A',
     protosort='A',
     startexp=0,
     color=1,
     coloredbars=1,
     uptime=1,
     totaluse=1,
     avuse=1,
     avsect=1,
     trapsect=1,
     baudsect=1,
     protosect=1,
     hourusage=1,
     dayusage=1,
     twolineuser=1,
     onetimecaller=1,
     usersect=1,
     nodesect=1,
     awards=1,
     oldgraph=0;

char tempname [L_tmpnam];        /* name of temp output file */
char temptrap [L_tmpnam];

char line[257],                              /* line that we are processing */
     word1[22],                                 /* first word of said line */
     word2[22],                                 /* second word. */
     lword1[31],                              /* first word of last line */
     lword2[31],                              /* second word */
     detect=0,
     *text;                                   /* pointer to text of message */

char mn[4];

int  chour[24],                                           /* calls per hour */
     cday[7],
     callbaud[16],
     numusr=0,          /* number of users that have called in this section */
     numnet=0,
     numproto=0,
     numexcl=0,
     backdays=0,
     expdays=0,
     j,
     i;

long date=0,                                         /* date of current line */
     ldate=0,              /* date of last line (both are seconds from '85) */
     fdate=0,
     odate=0,
     expdate=0,
     backdate=0,
     downtime=0,
     networktime=0,
     usedhour[24],
     usedbaud[16],
     usedday[7],
     usedtime=0,
     logins=0,
     hangups=0,
     chreq=0,
     crash=0,
     menter=0,
     mread=0,
     anon=0,
     newusr=0,
     wcup=0,
     wcdown=0,
     net=0,
     netcall=0,
     ofetch=0,
     incorp=0;

struct tm tfirst, tlast, t;     /* new time stuff. */

struct usr {                              /* The stuff for individual users */
     char name[31];
     unsigned excl:1;
     unsigned new:1;
     long ltin,
     timeon,
	  calls,
	  carrierd,
	  crash,
	  upload,
	  download,
          umread,
          umenter,
	  anon,
	  uchats;
     };

struct usr *user;

struct net {
     char name[31];
     unsigned excl:1;
     long timeon,
	  calls,
	  busy,
	  nmenter,
	  nmread,
	  nmduplicate,
	  nmexpired,
      crash,
	  files,
	  rooms;
	  };

struct net *network;

struct protocol {                /* protocol stuff. */
     char name[31];
     long down;
     long up;
     };

struct protocol *proto;

struct EXCLUSION
  {
    char excl [31];
  };

struct EXCLUSION *excltbl = 0;

static char *KEYWORDS[] = {
     "Login",             /* 0  Normal Login. followed by username */
     "New",               /* 1  Newuser Login. followed by username */
     "NetLogin",          /* 2  Net call in */
     "NetCallout",        /* 3  Net call out */
     "Carrier-Detect",    /* 4  Carrier Detected. followed by baud rate */
     "Chat",              /* 5  Chat request */
     "Citadel",           /* 6  Citadel terminated or started */
     "DragCit",  /* 7  Old. Citadel terminated or started...i forget which. */
     "Logout",            /* 8  Logout.  followed by username */
     "Carrier",           /* 9  Carrier dropped, used. */
     "Carrier-Loss",      /* 10 Also carrier dropped. not used, though. */
     "Anonymous",         /* 11 anonymous message. */
     "Before",            /* 12 date/time change */
     "After",             /* 13 date/time change */
     "Archive-view",      /* 14 View an arj file. */
     "Fetching",          /* 15 offline fetch. followed by "For [node]" */
     "Local"              /* 16 followed by "Net 6.9 Incorporation" */
     };

static char *NETKEYWORDS[] = {
	"NetDial",
	"Carrier-loss",
	"Carrier",
	"NetLogout",
    "Citadel",
    "Net6_9"
	};

static char *COLORS[] = {
     "b",                                   /* Red     */
     "c",                                   /* Green   */
     "d",                                   /* Yellow  */
     "e",                                   /* Blue    */
     "f",                                   /* Magenta */
     "g",                                   /* Cyan    */
     "h",                                   /* White   */
     "3a",                                 /* light gray     */
     "3b",                                 /* bright red     */
     "3c",                                 /* bright green   */
     "3d",                                 /* bright yellow  */
     "3e",                                 /* bright blue    */
     "3f",                                 /* bright magenta */
     "3g",                                 /* bright cyan    */
     "3h"                                  /* bright white   */
     };


static char *bauds[] = {
     "300",
     "1200",
     "2400",
     "4800",
     "7200",
     "9600",
     "12000",
     "14400",
     "16800",
     "19200",
     "28800",
     "38400",
     "57600",
     "115200",
     "230400",
     "con"
     };

static char *DAYS[] = {
     "Sunday",
     "Monday",
     "Tuesday",
     "Wednesday",
     "Thursday",
     "Friday",
     "Saturday",
     "Unknown"
     };

void main(int argc, char *argv[]);
void show_usage (char *cmd_path);
int readcfg (void);
int checkexcl (char *nym);
char sortopt (char *code);
void output(void);
void debug_out(void);
void login(void);
void netlogin(char type);
void getline(void);
void parseargs(int argc, char *argv[]);
void save (char *infile, char *outfile, int filter);
int  usortfunc(const void *this, const void *that);
int  nsortfunc(const void *this, const void *that);
int  psortfunc(const void *this, const void *that);

/****************************************************************************/
/* main() Sets up the files, does the main loop, and exits. Big deal.       */
/****************************************************************************/
void main(int argc, char *argv[])
{
int tfile;
char *ptr;
char *shit = malloc(10);

   printf("\n"
	  "TRAP.FBI Statistics Generator.\n"
	  "Version %s (%s %s)\n"
	  "By Captain James T Kirk (Audin Malmin) \n"
      "and modified by Counselor @ Blind Man's Bluff.  \n"
	  "\n",
	  version, __DATE__, __TIME__);

   if (argc < 3)
   {
       show_usage (argv [0]);
       exit(-1);
   }

   parseargs(argc, argv);

   if((tfile = open(argv[1], O_RDONLY)) == -1)
      {
      perror("Error: ");
      exit(-1);
      }

   traplength = filelength(tfile);

   close(tfile);

   /* sure would be nice if i could figure out a DECENT way to find the
      length of a file... */

   if ((in = fopen(argv[1], "r")) == NULL)
   {
       printf("\n Can't open \"%s\" for input.\n", argv[1]);
       exit(-1);
   }

   tmpnam (tempname);
   if ((out = fopen (tempname, "wt")) == NULL)
   {
       printf("\n Can't open \"%s\" for output.\n", tempname);
       fcloseall();
       exit(-1);
   }

   setvbuf (in, NULL, _IOFBF, 4096);

   if (expdays)
     {
       expdate = time (0) - (expdays * 86400);
       tmpnam (temptrap);
       if ((trap = fopen (temptrap, "wt")) == NULL)
       {
           printf("\n Can't open \"%s\" for output.\n", temptrap);
           fcloseall();
           exit(-1);
       }
     }

   if (backdays)
     {
       backdate = time (0) - (backdays * 86400);
     }

   for (i=0; i<24; i++)
       {
       chour[i]    = 0;
       usedhour[i] = 0;
       }

   for(i=0; i<7; i++)
      {
      cday[i]      = 0;
      usedday[i]   = 0;
      }

   for (i=0; i<16; i++)
   {
       usedbaud[i]=0;
       callbaud[i]=0;
   }

   word1[0]    = '\0';
   word2[0]    = '\0';
   lword1[0] = '\0';
   lword2[0] = '\0';

   printf("Reading \"%s\".\n", argv[1]);

					      /* calloc first struct member */
   if(!(user = (void *)calloc(1, sizeof(struct usr))))
      {
      printf("Out of memory...try supershelling first...\n");
      exit(-1);
      }

   if(!(proto = (void *)calloc(1, sizeof(struct protocol))))
     {
     printf("Out of memory...try supershelling first...\n");
     exit(-1);
     }

   if(!(network = (void *)calloc(1, sizeof(struct net))))
     {
     printf("Out of memory...try supershelling first...\n");
     exit(-1);
     }

   while (!feof (in))                              /* main loop starts here!! */
   {
       getline();

       if (expdays && !startexp && (date >= expdate))
         {
           startexp = 1;
           fprintf (trap, "%s\n", line);
         }

       if (backdays)
         {
           if (date < backdate)
             {
               fdate = 0;
               continue;
             }
         }

       for (i = 0; i < 16; i++)
           if (stricmp(word1, KEYWORDS[i])==0)
               break;

       switch(i)
       {
       case 0:   /* normal Login */
	       login();
	       break;
       case 1:   /* new user login */
	       newusr++;
	       login();
	       break;
       case 2:   /* net Login (not dialout) */
	       net++;
	       netlogin(1);
	       detect=0;
               break;
       case 3:   /* Net Dialout */
	       netcall++;
	       netlogin(2);
	       detect=0;
               break;
       case 4:   /* Carrier detect (baud) */
           if (word2 [0] == '\"')
             {
               if (ptr = strchr (text, '('))
                 {
                   strcpy (word2, ptr);
                 }
             }
	       for (j = 0; j < 15; j++)
		   {
		   sprintf (shit, "(%s)", bauds [j]);
		   if (stricmp (word2, shit) == 0)
		       break;
		   }
	       detect = 1;
	       break;
       case 5:   /* Chat request */
	       chreq++;
               break;
       case 6:   /* Citadel Started */
	       if ((numlines > 1) && (stricmp (word2, "Started") == 0))
		 {
		 downtime += date - ldate;
		 if(stricmp(lword2, "Terminated") != 0)
		   {
		   crash++;
		   printf("");
		   }
		 }
	       detect=0;
	       break;
       case 7:   /* "DragCit Started", unused */
       case 8:   /* "Logout", detected in the login() function */
       case 9:   /* "Carrier" Dropped, in login() funct */
      case 10:   /* "Carrier-Loss", unused */
	       detect=0;
	       break;
      case 11:   /* Anonymous message */
	       anon++;
	       break;
      case 12:   /* Before date/time change */
	       break;
      case 13:   /* After date/time change */
           odate += date - ldate;
	       break;
      case 15:   /* offline fetch */
	       ofetch++;
	       netlogin(0);
	       break;
      default:   /* a weird one.  (one we don't know how to handle) */
//	       detect=0;
	       break;
       }
   }

   /* do the sorting thing... */
   printf("\nSorting users by %c.\n", usersort);
   qsort(user, numusr, sizeof(struct usr), usortfunc);

   printf("Sorting protocols by %c.\n", protosort);
   qsort(proto, numproto, sizeof(struct protocol), psortfunc);

   printf("Sorting nodes by %c.\n", networksort);
   qsort(network, numnet, sizeof(struct net), nsortfunc);

   fdate += odate;
   if (downtime < 0)
       downtime = 0;

   printf("\nWriting \"%s\"\n", argv[2]);
   output();

   if(debug)
     debug_out();

   if (excltbl != 0)
       free (excltbl);
   free(user);
   free(proto);
   free(network);
   fcloseall();

   save (tempname, argv [2], !color);
   if (expdays)
       save (temptrap, argv [1], 0);

}

/****************************************************************************/
/* readcfg () reads the config file and builds the exclusion table.         */
/****************************************************************************/
int readcfg ()
  {
    int i, j;
    int keywords = 8;
    int sections = 13;
    FILE *cfg;
    static char *keyword [] = 
      {
        "exclude",
        "omit",
        "color",
        "mincalls",
        "verbos",
        "expire",
        "graphbars",
        "sort"
      };
    static char *section [] = 
      {
        "uptime",
        "totals",
        "usage",
        "averages",
        "trap",
        "baud",
        "proto",
        "hours",
        "days",
        "firstime",
        "users",
        "nodes",
        "awards"
      };

    if ((cfg = fopen (configfile, "rt")) == 0)
      {
        printf ("No config file present.  \n\n");
        return 0;
      }

    printf ("Reading configuration:  %s.  \n", configfile);
    while (!feof (cfg))
      {
        text = fgets (line, sizeof (line), cfg);
        if (text == 0) break;
        if (line [0] != '#') continue;

        i = sscanf (line, "%s \"%30[^\"\n\r\t]", word1, lword1);
        if (i == 1)
            i = sscanf (line, "%s %30s", word1, lword1);
        if (i != 2) continue;

        for (i = 0; i < keywords; i++)
            if (stricmp (&word1 [1], keyword [i]) == 0) 
                break;

        switch (i)
          {
          case 0:       // Exclude
            numexcl++;
            if (!(excltbl = realloc (excltbl, sizeof (struct EXCLUSION) * numexcl)))
              {
                printf ("Out of memory.  Unable to store %s.", lword1);
                return 0;
              }

            strcpy (excltbl [numexcl-1].excl, lword1);
            printf ("Excluding %s.  \n", excltbl [numexcl-1].excl);
            break;

          case 1:       // Omit
            for (j = 0; j < sections; j++)
                if (stricmp (lword1, section [j]) == 0)
                    break;

            switch (j)
              {
              case 0:
                uptime = 0;
                break;

              case 1:
                totaluse = 0;
                break;

              case 2:
                avuse = 0;
                break;

              case 3:
                avsect = 0;
                break;

              case 4:
                trapsect = 0;
                break;

              case 5:
                baudsect = 0;
                break;

              case 6:
                protosect = 0;
                break;

              case 7:
                hourusage = 0;
                break;

              case 8:
                dayusage = 0;
                break;

              case 9:
                onetimecaller = 0;
                break;

              case 10:
                usersect = 0;
                break;

              case 11:
                nodesect = 0;
                break;

              case 12:
                awards = 0;
                break;

              default:
                printf ("Report section '%s' unknown.  \n", lword1);
                break;
              }
            if (j < sections)
                printf ("Omitting %s section.  \n", lword1);
            break;

          case 2:       // Color
            if ((stricmp (lword1, "on") == 0) || atoi (lword1))
              {
                color=1;
                printf ("Color turned on.  \n");
              }
            else if ((stricmp (lword1, "off") == 0) || (lword1 [0] == '0'))
              {
                color=0;
                printf ("Color turned off.  \n");
              }
            break;

          case 3:       // Mincalls
            seeuser = atol (lword1);
            printf ("Users having made %ld calls or more will be reported.  \n", seeuser+1);
            break;

          case 4:       // Verbos
            if ((stricmp (lword1, "on") == 0) || atoi (lword1))
                twolineuser = 1;
            else if ((stricmp (lword1, "off") == 0) || (lword1 [0] == '0'))
                twolineuser = 0;
            printf ("Verbos is %s.  \n", lword1);
            break;

          case 5:    // Expire
            expdays = atoi (lword1);
            if (expdays > 0)
                printf ("Trap file will be shrunk to the most recent %d days.  \n", expdays);
            else
              {
                expdays = 0;
                printf ("Trap file will not be shrunk.  \n");
              }
            break;

          case 6:       // Graphbars
            if ((stricmp (lword1, "color") == 0))
              {
                coloredbars=1;
                printf ("Graph bar color turned on.  \n");
              }
            else if ((stricmp (lword1, "white") == 0))
              {
                coloredbars=0;
                printf ("Graph bar color turned off.  \n");
              }
            break;

          case 7:       // Sort
            if (strlen (lword1) == 3)
                if (!sortopt (lword1))
                    printf ("Sort options set to %s.  \n", lword1);
            break;

          default:
            printf ("Keyword '%s' unknown.  \n", word1);
            break;
          }
      }

    fclose (cfg);
    printf ("\n");

    return 1;
  }


/****************************************************************************/
/* checkexcl (nym) looks up nym in the exclusion table, returning true if   */
/* found, false if not.                                                     */
/****************************************************************************/
  int checkexcl (char *nym)
    {
    int i;

    for (i = 0; i < numexcl; i++)
        if ((stricmp (nym, excltbl [i].excl)) == 0)
            return 1;

    return 0;
    }

/****************************************************************************/
/* sortopt () sets the sort options for users, networks and protocols       */
/****************************************************************************/
char sortopt (char *code)
  {
    char c, badopt = 0;

    switch (c = (char) toupper(code [0]))
      {
      case 'A':
      case 'T':
      case 'M':
      case 'L':
      case 'R':
      case 'E':
      case 'P':
      case 'U':
      case 'D':
      case 'C':
        usersort=c;
        break;
      default:
        printf ("Bad user sort option \"%c\".  \n", code [0]);
        badopt = 1;
        break;
      }
    switch (c = (char) toupper (code [1]))
      {
      case 'A':   /* alpha */
      case 'B':   /* busy signals */
      case 'C':   /* calls */
      case 'D':   /* duplicates */
      case 'E':   /* expired */
      case 'P':   /* posts */
      case 'R':   /* read */
      case 'T':   /* time on */
        networksort=c;
        break;
      default:
        printf ("Bad network sort option \"%c\".  \n", code [1]);
        badopt = 1;
        break;
      }
    switch (c = (char) toupper (code [2]))
      {
      case 'A':
      case 'U':
      case 'D':
        protosort=c;
        break;
      default:
        printf ("Bad protocol sort option \"%c\".  \n", code [2]);
        badopt = 1;
        break;
      }

    return badopt;
  }

/****************************************************************************/
/* sortfunc() takes 3rd command line arg and uses it for sorting option     */
/****************************************************************************/
int usortfunc(const void *this1, const void *that1)
{
    const struct usr *this = this1;
    const struct usr *that = that1;
    long valret=0;

    switch(usersort)
    {
    case 'A':
	valret = 0;
	break;
    case 'T':
	valret = that->timeon - this->timeon;
	break;
    case 'M':
	valret = that->ltin - this->ltin;
	break;
    case 'L':
	valret = that->calls - this->calls;
	break;
    case 'R':
	valret = that->umread - this->umread;
	break;
    case 'E':
	valret = that->umenter - this->umenter;
	break;
    case 'P':
	valret = (that->umenter/that->calls) - (this->umenter/this->calls);
	break;
    case 'U':
	valret = that->upload - this->upload;
	break;
    case 'D':
	valret = that->download - this->download;
	break;
    case 'C':
	valret = that->uchats - this->uchats;
	break;

    default:
	valret = 0;
	break;
    }

    if(!valret)
	valret = stricmp(this->name, that->name);

    if (valret > 0) return 1;
    if (valret < 0) return -1;
    return 0;
}

int nsortfunc(const void *this1, const void *that1)
{
    const struct net *this = this1;
    const struct net *that = that1;
    long valret=0;

    switch(networksort)
    {
    case 'A':
	valret = 0;
	break;
    case 'B':
	valret = that->busy - this->busy;
	break;
    case 'C':
	valret = that->calls - this->calls;
	break;
    case 'R':
	valret = that->nmread - this->nmread;
	break;
    case 'P':
	valret = that->nmenter - this->nmenter;
	break;
    case 'E':
	valret = that->nmexpired - this->nmexpired;
	break;
    case 'T':
	valret = that->timeon - this->timeon;
	break;
    default:
	valret = 0;
	break;
    }

    if(!valret)
	valret = stricmp(this->name, that->name);

    if (valret > 0) return 1;
    if (valret < 0) return -1;
    return 0;
}
int psortfunc(const void *this1, const void *that1)
{
    const struct protocol *this = this1;
    const struct protocol *that = that1;
    long valret=0;

    switch(protosort)
    {
    case 'A':
	valret = 0;
	break;
    case 'U':
	valret = that->up - this->up;
	break;
    case 'D':
	valret = that->down - this->down;
	break;
    default:
	valret = 0;
	break;
    }

    if(!valret)
	valret = stricmp(this->name, that->name);

    if (valret > 0) return 1;
    if (valret < 0) return -1;
    return 0;
}

/****************************************************************************/
/* getline()   Gets a line from the trap file and sets the date and ldate   */
/*             Also sets word1, and text.                                   */
/****************************************************************************/
void getline(void)
{
   int loop;
   static char *mnts[] = {                                /* All the months */
		   "Jan", "Feb", "Mar", "Apr", "May", "Jun",
		   "Jul", "Aug", "Sep", "Oct", "Nov", "Dec" };

   ldate=date;

   fgets(line, 256, in);                                    /* Get the line */
   line[strlen(line)-1]='\0';                            /* snip the return */

   numlines++;

   sscanf(line, "%d%3s%d %d:%d:%d", &(t.tm_year), mn, &(t.tm_mday),
	   &(t.tm_hour), &(t.tm_min), &(t.tm_sec));

   for (loop=0; loop<12; loop++)
       if (strcmp(mn, mnts[loop])==0)
	   t.tm_mon=loop;

   date=mktime(&t);

   if (startexp && !feof (in))
       fprintf (trap, "%s\n", line);

   if (!fdate)
      {
        if (t.tm_isdst)
          {
            t.tm_hour--;
            date -= 3600;
          }
       memcpy(&tfirst, &t, sizeof(t));
       fdate=date;
      }

   memcpy(&tlast, &t, sizeof(t));

   text=(line + 19);                                /* set the text segment */

   lword1[0] = '\0';
   lword2[0] = '\0';

   strcpy(lword1, word1);
   strcpy(lword2, word2);

   word1[0] = '\0';
   word2[0] = '\0';

   sscanf(text, "%s %s", word1, word2);                    /* Get the words */

}

/****************************************************************************/
/* login() Handles all things produced while a user is logged in.           */
/****************************************************************************/
void login(void)
{
   int me, mr, logout=0;
   long tin, tout, toff;       /* time in, time out */
   int usrnum, lp, protonum;
   char *nym;

   printf("");
   tin=date;
   me    =0;
   mr    =0;
   logins++;

   if (stricmp(word1, KEYWORDS[1])==0)
       nym=(text+9);
   else
       nym=(text+6);

   for (lp=0, usrnum=-1; lp<numusr; lp++)
       if (stricmp(nym, user[lp].name)==0)
           usrnum=lp;

   if (usrnum==-1)
   {
       numusr++;
       usrnum=numusr-1;

			   /* realloc for new user, only as many as we need */
       if(!(user = (void *)realloc((void *)user, numusr * sizeof(struct usr))))
	 {
	 printf("Damn...I ran out of memory...  you'd better try a supershell.\n");
	 exit(-1);
	 }

       strcpy(user[usrnum].name, nym);

       if (stricmp(word1, KEYWORDS[1]) == 0)
           user[usrnum].new = 1;
       else
           user[usrnum].new = 0;

       user[usrnum].excl     = checkexcl (nym);
       user[usrnum].ltin     = 0;
       user[usrnum].timeon   = 0;
       user[usrnum].calls    = 0;
       user[usrnum].carrierd = 0;
       user[usrnum].upload   = 0;
       user[usrnum].download = 0;
       user[usrnum].umread   = 0;
       user[usrnum].umenter  = 0;
       user[usrnum].anon     = 0;
       user[usrnum].crash    = 0;
       user[usrnum].uchats   = 0;
   }

   chour[t.tm_hour]++;

   cday[t.tm_wday]++;

   while(!feof(in) && !logout)  /* loop starts here */
   {
       getline();

       for (i = 5; i < 14; i++)
	   if (stricmp(word1, KEYWORDS[i])==0)
	       break;

       switch(i)
       {
       case 5: /* chat */
	       user[usrnum].uchats++;
	       chreq++;
	       break;
       case 6: /* Citadel Started?  yikes!  a crash! */
	       if(stricmp(word2, "Started")==0)
		 {
		 downtime += date - ldate;
		 crash++;
		 user[usrnum].crash++;
		 printf("");
		 }
	       logout=1;
	       tin=date;
	       break;
       case 7: /* Dragcit ...  something...  not really used. */
	       logout=1;
	       tin=date;
	       break;
       case 8: /* Logout */
/*	       if(stricmp(text+7, user[usrnum].name) != 0)
		 {
		 strcpy(user[usrnum].name, text+7);
		 }
 */
	       if (!feof(in)) /* get messages entered */
	       {
		   getline();
                   sscanf(text, "%s %d", word1, &me);
               }
	       if (!feof(in)) /* get messages read */
	       {
		   getline();
                   sscanf(text, "%s %d", word1, &mr);
               }
	       logout=1;
	       break;
       case 9: /* Carrier Dropped */
	       if(stricmp(word2, "dropped")==0)
		 {
		 hangups++;
		 user[usrnum].carrierd++;
		 }
	       break;
       case 10: /* Carrier-Loss */
	       break;
       case 11:
		anon++;
		user[usrnum].anon++;
		break;
       case 12:     // Before date/time change
		break;
       case 13:     // After date/time change
        toff = date - ldate;
        tin += toff;
        odate += toff;
		break;
       default:
	       if(stricmp(word2, "download")==0)
	       {
		   wcdown++;
		   user[usrnum].download++;

		   for(lp=0, protonum=-1; lp<numproto; lp++)
		      if(stricmp(word1, proto[lp].name)==0)
			protonum=lp;

		   if(protonum==-1)
		   {
		     numproto++;
		     protonum=numproto-1;

		     proto = (void *)realloc((void *)proto, numproto * sizeof(struct protocol));
		     strcpy(proto[protonum].name, word1);

		     proto[protonum].down = 0;
		     proto[protonum].up = 0;
		   }
		   proto[protonum].down++;
	       }
	       else
		   if(stricmp(word2, "upload")==0)
		       {
		       wcup++;
		       user[usrnum].upload++;

		       for(lp=0, protonum=-1; lp<numproto; lp++)
			 if(stricmp(word1, proto[lp].name)==0)
			   protonum=lp;

		       if(protonum==-1)
			 {
			 numproto++;
			 protonum=numproto-1;

			 proto = (void *)realloc((void *)proto, numproto * sizeof(struct protocol));

			 strcpy(proto[protonum].name, word1);
			 proto[protonum].down = 0;
			 proto[protonum].up = 0;
			 }
		       proto[protonum].up++;
		       }
	       break;
       }
   }

       if (date < tin)
         {    // Protection against date/time change -- should never happen
           odate += date - tin;
           tin = date;
         }

   tout                   = date;
   usedtime              += (tout-tin);
   usedhour[t.tm_hour]   += (tout-tin);
   usedday[t.tm_wday]    += (tout-tin);

   if(detect)
   {
       callbaud[j]++;
       usedbaud[j]       += (tout-tin);
       detect=0;
   }
   else
   {
       callbaud[15]++;
       usedbaud[15]       += (tout-tin);
   }

   menter                += me;
   mread                 += mr;
   user[usrnum].ltin     = tin;
   user[usrnum].timeon   += (tout-tin);
   user[usrnum].calls++;

   if(user[usrnum].calls > 1 && user[usrnum].new == 1)
      user[usrnum].new = 0;

   user[usrnum].umread   += mr;
   user[usrnum].umenter  += me;
}

/****************************************************************************/
/* netlogin (char type) handles network stuff.                              */
/*                type=0  prefetch                                          */
/*                type=1  NetLogin                                          */
/*                type=2  NetCallout                                        */
/****************************************************************************/
void netlogin (char type)
{
   int i, me, mr, mx, md, logout;
   long tin, tout;
   int netnum, lp;
   char *nym;

   printf("");
   tin = date;
   me  = 0;
   mr  = 0;
   mx  = 0;
   md  = 0;
   logout = 0;

   if (type > 0)
     nym=text+strlen(word1)+1;
   else if (type == 0)
       {
       nym=text+14;
       nym[strlen(nym)-1] = '\0'; /* kill the end quotation mark */
       }

   for (lp=0, netnum=-1; lp<numnet; lp++)
       if (stricmp(nym, network[lp].name)==0)
	   netnum=lp;

   if (netnum==-1)
   {
       numnet++;
       netnum=numnet-1;

       network = (void *)realloc((void *)network, numnet * sizeof(struct net));

       strcpy(network[netnum].name, nym);

       network[netnum].excl        = checkexcl (nym);
       network[netnum].timeon      = 0;
       network[netnum].calls       = 0;
       network[netnum].busy	   = 0;
       network[netnum].nmread      = 0;
       network[netnum].nmenter     = 0;
       network[netnum].nmduplicate = 0;
       network[netnum].nmexpired   = 0;
       network[netnum].crash	   = 0;
       network[netnum].files	   = 0;
       network[netnum].rooms	   = 0;
   }

   if (type == 0)
     return;

   while(!logout && !feof(in))
     {
     getline();

     for (i = 0; i < 6; i++)
	  if(stricmp(word1, NETKEYWORDS[i])==0)
	    break;

     switch(i)
     {
     case 0:    // NetDial
	    if(stricmp(word2, "(busy)")==0)
	      {
	      network[netnum].busy++;
	      logout=1;
	      }
	    break;
     case 1:    // Carrier-Loss
	    break;
     case 2:    // Carrier
//	    hangups++;    // Do we really want to do this?  Nodes always hang up.  
	    break;
     case 3:    // NetLogout
	    if ((type == 1) || (strnicmp (text+10+strlen (network [netnum].name), 
           " (succeeded)", 7) == 0))
	    {
	      if (!feof (in))
	      {
		getline();
		sscanf(text, "%s %d", word1, &me);
	      }
	      if (!feof (in))
	      {
		getline();
		sscanf(text, "%s %d", word1, &mr);
	      }
	      if (!feof (in))
	      {
		getline();
		sscanf(text, "%s %d", word1, &mx);
	      }
	      if (!feof (in))
	      {
		getline();
		sscanf(text, "%s %d", word1, &md);
	      }
	    }
	    logout=1;
	    break;
     case 4:    // Citadel
       if(stricmp(word2, "Started")==0)
		 {
		 downtime += date - ldate;
		 crash++;
         network[netnum].crash++;
		 printf("");
		 }
	    logout=1;
        tin=date;
	    break;
     case 5:    // Net6_9 (MavenCit)
        network[netnum].busy++;
        logout=1;
	    break;
     default:
	    break;
     }
     }

   tout = date;
   networktime += (tout-tin);

   network[netnum].timeon      += (tout-tin);
   network[netnum].calls++;
   network[netnum].nmenter     += me;
   network[netnum].nmread      += mr;
   network[netnum].nmduplicate += md;
   network[netnum].nmexpired   += mx;
}

/****************************************************************************/
/* show_usage (char *cmd_path) Displays a help screen.                                    */
/****************************************************************************/
void show_usage (char *cmd_path)
  {
    char drive[3], dir[80], fname[9], ext[5];

    _splitpath (cmd_path, drive, dir, fname, ext);
    printf ("Usage:  %s trap_file output_file [config_file] [options]\n"
        "\n"
        "    trap_file:  the path and name of Citadel's Trapfile.\n"
        "  output_file:  path and filename for output.\n"
        "  config_file:  path and filename for an optional configuration file.\n"
        "      options:\n"
        "      -1: First time caller section. -1+ = included, -1- = excluded.\n"
        "      -2: Two line userlist.  -2+ = on (default), -2- off.\n"
        "     -Bn: Start back n days from current date.\n"
        "      -C: Color in output file. -C+ = on (default), -C- = off.\n"
        "      -D: Debug mode.\n"
        "      -G: Graph bars. -G+ = multi-colored bars (default), -G- = white bars.\n"
        "      -U: User/node section.  -U+ = included (default), -U- = excluded.\n"
        "     -Xn: Expire trap file entries before n days.  0 (default) no expiration. \n"
        "     -#n: Only users with > n calls get listed in userlist. (-#1 default)\n",
        fname);
    printf ("\n== Press any key to continue => ");
    getch ();
    printf ("\n"
        "   -Sunp: Sort(user)(network)(protocol)\n"
        "          user:\n"
        "               A)lphabetic, T)ime, CalL)s, Messages R)ead, Messages E)ntered,\n"
        "               M)ost recent login, \n"
        "               P)ost/Call Ratio, U)ploads, D)ownloads, C)hat Requests.\n"
        "       network:\n"
        "               A)lphabetic, B)usy Signals, C)alls, Messages D)uplicated,\n"
        "               Messages E)xpired, P)osts, Messages R)ead, T)ime.\n"
        "      protocol:\n"
        "               A)lphabetic, U)ploads, D)ownloads.\n"
        "          eg: \"-Sacu\"  for Alpha, calls, uploads.\n");

    return;
  }


/****************************************************************************/
/* output() Have fun with this part, it could be made to say just about     */
/*          anything you want.                                              */
/****************************************************************************/
void output (void)
  {
    float temp, temps [24];
    int loop1, loop2;
    int j;

    if (uptime)
      {
        fprintf (out, " 53e      From                  To               Total time\n");
        fprintf (out, " a       0\n");
        fprintf (out, " %2d3a/0%02d3a/0%d %2d3a:0%02d3a:0%02d -- %2d3a/0%02d3a/0%d %2d3a:0%02d3a:0%02d =  %2ld days3a,0 %2ld3a:0%02ld3a:0%02ld\n \n",
            tfirst.tm_mon+1, tfirst.tm_mday, tfirst.tm_year,
            tfirst.tm_hour, tfirst.tm_min, tfirst.tm_sec,
            tlast.tm_mon+1, tlast.tm_mday, tlast.tm_year,
            tlast.tm_hour, tlast.tm_min, tlast.tm_sec,
            (date-fdate)/60/60/24,
            (date-fdate)/60/60 % 24,
            (date-fdate)/60 % 60,
            (date-fdate) % 60);
      }

    if (totaluse)
      {
        fprintf (out, " 3e    Used time         Network time     Total busy time    Total down time\n");
        fprintf (out, " a      0\n");
        fprintf (out, " %2ld days3a,0 %2ld3a:0%02ld3a:0%02ld  %2ld days3a,0 %2ld3a:0%02ld3a:0%02ld  %2ld days3a,0 %2ld3a:0%02ld3a:0%02ld  %2ld days3a,0 %2ld3a:0%02ld3a:0%02ld\n",
            (usedtime)/60/60/24,
            (usedtime)/60/60 % 24,
            (usedtime)/60 % 60,
            (usedtime) % 60,
            (networktime)/60/60/24,
            (networktime)/60/60 % 24,
            (networktime)/60 % 60,
            (networktime) % 60,
            (usedtime+networktime)/60/60/24,
            (usedtime+networktime)/60/60 % 24,
            (usedtime+networktime)/60 % 60,
            (usedtime+networktime) % 60,
            (downtime)/60/60/24,
            (downtime)/60/60 % 24,
            (downtime)/60 % 60,
            (downtime) % 60);

        fprintf (out, " \n");
      }

    if (avuse)
      {
        fprintf (out, " 3eAverage Use  Average Timea/eCall\n");
        fprintf (out, " a  0\n");
        fprintf (out, "   %5.02f3a%%0        %2ld3a:0%02ld3a:0%02ld\n",
            (float)usedtime/(date-fdate)*100,
            (usedtime/logins)/60/60 % 24,
            (usedtime/logins)/60 % 60,
            (usedtime/logins) % 60);

        fprintf (out, " \n");
      }

    temp = (float)(date-fdate)/60/60/24;

    if (avsect)
      {
        fprintf (out, " 3eAverages pera:e          call       day     period\n");
        fprintf (out, " a                         \n");

        fprintf (out, " eCallsa..............:  0   %8.02f  %8d\n",
                (float)logins/((float)(date-fdate)/60/60/24),
                           logins);
        if (menter || mread || anon)
            fprintf (out, " 3eMessagesa...........:0\n");
        if (menter)
            fprintf (out, " 3e   Entereda.........:0 %8.2f  %8.2f  %8ld\n",
                           ((float)menter/logins),
                           ((float)menter/temp),
                           menter);
        if (mread)
            fprintf (out, " 3e   Reada............:0 %8.2f  %8.2f  %8ld\n",
                           ((float)mread/logins),
                           ((float)mread/temp),
                           mread);
        if (anon)
            fprintf (out, " 3e   Anonymousa.......:0 %8.2f  %8.2f  %8ld\n",
                           ((float)anon/logins),
                           ((float)anon/temp),
                           anon);
        if (wcup || wcdown)
            fprintf (out, " 3eFilesa..............:0\n");
        if (wcup)
            fprintf (out, " 3e   Uploadeda........:0 %8.2f  %8.2f  %8ld\n",
                           ((float)wcup/logins),
                           ((float)wcup/temp),
                           wcup);
        if (wcdown)
            fprintf (out, " 3e   Downloadeda......:0 %8.2f  %8.2f  %8ld\n",
                           ((float)wcdown/logins),
                           ((float)wcdown/temp),
                           wcdown);
        if (chreq)
            fprintf (out, " 3eChatsa..............:0 %8.2f  %8.2f  %8ld\n",
                           ((float)chreq/logins),
                           ((float)chreq/temp),
                           chreq);
        if (hangups)
            fprintf (out, " 3eDropped Carriersa...:0 %8.2f  %8.2f  %8ld\n",
                           ((float)hangups/logins),
                           ((float)hangups/temp),
                           hangups);
        if (newusr)
            fprintf (out, " 3eNew Usersa..........:0 %8.2f  %8.2f  %8ld\n",
                           ((float)newusr/logins),
                           ((float)newusr/temp),
                           newusr);
        if (net || netcall)
        fprintf (out, " 3eNetworkinga.........:0\n");
        if (netcall)
            fprintf (out, " 3e    Net calloutsa...:0 %8.2f  %8.2f  %8ld\n",
                           ((float)netcall/logins),
                           ((float)netcall/temp),
                           netcall);
        if (net)
            fprintf (out, " 3e    Net callinsa....:0 %8.2f  %8.2f  %8ld\n",
                           ((float)net/logins),
                           ((float)net/temp),
                           net);
        if (net || netcall)
            fprintf (out, " 3e    Total Networksa.:0 %8.2f  %8.2f  %8ld\n",
                           ((float)(net+netcall)/logins),
                           ((float)(net+netcall)/temp),
                           (net+netcall));
        if (ofetch)
            fprintf (out, " 3e    Offline fetchesa:0 %8.2f  %8.2f  %8ld\n",
                           ((float)ofetch/logins),
                           ((float)ofetch/temp),
                           ofetch);
        if (crash)
            fprintf (out, " 3eCrashesa............:0 %8.2f  %8.2f  %8ld\n",
                           ((float) crash/logins),
                           ((float) crash/temp),
                           crash);
        fprintf (out, " \n");
      }

    if (trapsect)
      {
        fprintf (out, " 3eTrapfile\n");
        fprintf (out, " a\n");
        fprintf (out, " eSizea.............:0  %ld bytes\n", traplength);
        fprintf (out, " 3eLinesa............:0  %ld\n", numlines);
        fprintf (out, " 3eAverage lines/daya:0  %-8.2f\n", (float)numlines/temp);
        fprintf (out, " \n");
      }

    if (baudsect || protosect)
      {
        fprintf (out, " 3e");
        if (baudsect)
            fprintf (out, "  Baud logins  usage     ");
        if (protosect)
            fprintf (out, "Protocol                     up  down");
        fprintf (out, "\n a");
        if (baudsect)
            fprintf (out, "       ");
        if (protosect)
            fprintf (out, "  ");
        fprintf (out, "0\n");

        i = 0;
        j = 0;
        if (!protosect)
            j = numproto;
        while (j < numproto || i < 16)
          {
            if (baudsect)
              {
                while (!callbaud[i] && i<16) /* scroll through bauds untill we find a used one */
                    i++;
                if(i<16) /* are there any bauds left? */
                  {
                    fprintf (out, " %6s %6d %5.02f3a%%", 
                        bauds [i], callbaud [i],
                        ((float) usedbaud [i] / (float) (usedtime) * 100));
                  }
                else /* out of baud rates...just put in spaces to line up protocol. */
                    fprintf (out, "                     3a");
                if (protosect)
                    fprintf (out, "   ");
                fprintf (out, "0");
              }
            else
                i = 16;

            if (j < numproto) /* any protocols left? */
              {
                fprintf (out, " %-25.25s %5ld %5ld\n",
                    proto[j].name, proto[j].up, proto[j].down);
              }
            else /* nope...just put in a CR */
                fprintf (out, "\n");

            i++;
            j++;
          }

        fprintf (out, " \n");
      }

            // Charts

    if (hourusage)
      {
        fprintf (out, " 3ePercent Usage a(eby houra)\n");
        fprintf (out, " 0\n");

        j = 1;
        for (loop1 = 0; loop1 < 24; loop1++)
          {
            temps [loop1] = (float) usedhour [loop1] / (usedtime) * 100;
            if (temps [loop1] > 50)
                j = 2;
          }

        i = 0;
        for (loop1 = 0; loop1 < 24; loop1++)
          {
            fprintf (out, " 3e%02da (e%05.2f%%a):0 ", loop1, temps [loop1]);

            for (loop2 = 0; loop2 < (int) (temps [loop1] / j); loop2++)
              {
                if (coloredbars)
                    fprintf (out, "%s", COLORS[i]);
                else
                    fprintf (out, "");

                if (i>12)
                    i = -1;
                i++;
              }

            fprintf (out, "\n");
          }

        fprintf (out, " \n");
      }

    if (dayusage)
      {
        fprintf (out, " 3ePercent Usage a(eby daya)\n");
        fprintf (out, " 0\n");

        j = 1;
        for (loop1 = 0; loop1 < 7; loop1++)
          {
            temps [loop1] = (float) usedday [loop1] / (usedtime) * 100;
            if (temps [loop1] > 50)
                j = 2;
          }

        i = 0;
        for (loop1 = 0; loop1 < 7; loop1++)
          {
            fprintf (out, " 3e%-9sa (e%05.2f%%a): ", DAYS [loop1], 
                temps [loop1]);

            for (loop2 = 0; loop2 < (int) (temps [loop1] / j); loop2++)
              {
                if (coloredbars)
                    fprintf (out, "%s", COLORS [i]);
                else
                    fprintf (out, "");

                if (i>12)
                    i = -1;
                i++;
              }

            fprintf (out, "\n");
          }

        fprintf (out, " \n");
      }

        // User data

    if (usersect)
      {
        if (twolineuser)
          {
            fprintf (out, " 3eUsername                  time  calls  drop   post   p/c  anon  u/l\n");
            fprintf (out,     "                       time/call %%usage crash  read        chat  d/l\n");
            fprintf (out,   " a       0\n");
          }
        else
          {
            fprintf (out, " 3eUsername                  time  calls chat  read  post   p/c anon  u/l  d/l\n");
            fprintf (out, " a         0\n");
          }

        for (loop1=0; loop1<numusr; loop1++)
            if (!user [loop1].excl && user [loop1].calls>seeuser)
              {
                if (twolineuser)
                  {
                    fprintf (out, " %-24.24s %3ld:%02ld  %5ld %5ld %6ld %5.2f %4ld %4ld\n",
                        user [loop1].name, user [loop1].timeon / 60L / 60L,
                        user [loop1].timeon / 60L % 60L, user [loop1].calls,
                        user [loop1].carrierd, user [loop1].umenter,
                        ((float) user [loop1].umenter / user [loop1].calls), 
                        user [loop1].anon,
                        user [loop1].upload);
                    fprintf (out, "                           %2ld:%02ld %6.2f %5ld %6ld       %4ld %4ld\n",
                        ((user [loop1].timeon / user [loop1].calls) / 60L / 60L),
                        ((user [loop1].timeon / user [loop1].calls) / 60L % 60L),
                        ((float) user [loop1].timeon / usedtime * 100),
                        user [loop1].crash, 
                        user [loop1].umread, 
                        user [loop1].uchats,
                        user [loop1].download);
                  }
                else
                    fprintf (out, " %-24.24s %3ld:%02ld %5ld %4ld %6ld %4ld %5.2f %4ld %4ld %4ld\n",
                        user [loop1].name, user [loop1].timeon / 60L / 60L,
                        user [loop1].timeon / 60L % 60L, user [loop1].calls,
                        user [loop1].uchats, 
                        user [loop1].umread, user [loop1].umenter,
                        ((float) user [loop1].umenter / user [loop1].calls), 
                        user [loop1].anon,
                        user [loop1].upload, user [loop1].download);
              }

        fprintf (out, " \n");
      }

        // Net data

    if (nodesect)
      {
        fprintf (out, " 3eNodename                   time  calls busy  read  post   dup   exp crash\n");
        fprintf (out, " a        0\n");
        for (loop1=0; loop1<numnet; loop1++)
            if (!network [loop1].excl)
                fprintf (out, " %-25.25s %3ld3a:0%02ld %5ld %4ld %5ld %5ld %5ld %5ld %5ld\n",
                    network[loop1].name, network[loop1].timeon/60/60,
                    network[loop1].timeon/60 % 60, network[loop1].calls,
                    network[loop1].busy, network[loop1].nmread,
                    network[loop1].nmenter, network[loop1].nmduplicate,
                    network[loop1].nmexpired, network[loop1].crash);

        fprintf (out, " \n");
      }

    if (newusr && onetimecaller)
      {
        fprintf (out, " 3eFirst time callers\n");
        fprintf (out, " a0\n");
        for (loop1=0,loop2=0; loop1<numusr; loop1++)
            if (!user [loop1].excl && user [loop1].new)
              {
                fprintf (out, " %-24.24s ", user[loop1].name);
                loop2++;
                if (loop2 % 3==0)
                    fprintf (out, "\n");
              }
        if (loop2 % 3==0)
            fprintf (out, " \n");
        else
            fprintf (out, "\n \n");
      }

    if (awards)
      {
        for (j = 0; j < numusr; j++)
            if (!user [j].excl) break;
        if (j == numusr) return;

        fprintf (out, " 3eAwards\n");
        fprintf (out, " a0\n");

        for (loop1=j, loop2=j; loop1<numusr; loop1++)
            if (!user [loop1].excl && (user[loop1].umenter > user[loop2].umenter))
                loop2=loop1;

        if (user [loop2].umenter)
            fprintf (out, " 3eMost messages entereda..:0  %s with %ld of um.\n",
                user[loop2].name, user[loop2].umenter);

        for (loop1=j, loop2=j; loop1<numusr; loop1++)
            if (!user [loop1].excl && ((user[loop1].umenter/user[loop1].calls*10) 
               > (user[loop2].umenter/user[loop2].calls*10)))
                loop2=loop1;

        if (user [loop2].umenter)
            fprintf (out, " 3eMost posts per calla....:0  %s with %5.2f.\n",
                user[loop2].name, (float)user[loop2].umenter/user[loop2].calls);

        for (loop1=j, loop2=j; loop1<numusr; loop1++)
            if (!user [loop1].excl && (user[loop1].umread > user[loop2].umread))
                loop2=loop1;

        if (user [loop2].umread)
            fprintf (out, " 3eMost messages reada.....:0  %s with a whopping %ld.\n",
                user[loop2].name, user[loop2].umread);

        for (loop1=j, loop2=j; loop1<numusr; loop1++)
            if (!user [loop1].excl && (user[loop1].anon > user[loop2].anon))
                loop2=loop1;

        if (user [loop2].anon)
            fprintf (out, " 3eMost Anonymous messagesa:0  %s with %ld.\n",
                user[loop2].name, user[loop2].anon);

        for (loop1=j, loop2=j; loop1<numusr; loop1++)
            if (!user [loop1].excl && (user[loop1].upload > user[loop2].upload))
                loop2=loop1;

        if (user [loop2].upload)
            fprintf (out, " 3eMost files uploadeda....:0  %s with %ld uploads.\n",
                user[loop2].name, user[loop2].upload);

        for (loop1=j, loop2=j; loop1<numusr; loop1++)
            if (!user [loop1].excl && (user[loop1].download > user[loop2].download))
                loop2=loop1;

        if (user [loop2].download)
            fprintf (out, " 3eMost files downloadeda..:0  %s with %ld leeches.\n",
                user[loop2].name, user[loop2].download);

        for (loop1=j, loop2=j; loop1<numusr; loop1++)
            if (!user [loop1].excl && (user[loop1].uchats > user[loop2].uchats))
                loop2=loop1;

        if (user [loop2].uchats)
            fprintf (out, " 3eMost chat requestsa.....:0  %s with %ld annoying chats.\n",
                user[loop2].name, user[loop2].uchats);

        for (loop1=j, loop2=j; loop1<numusr; loop1++)
            if (!user [loop1].excl && (user[loop1].timeon > user[loop2].timeon))
                loop2=loop1;

        if (user [loop2].timeon)
            fprintf (out, " 3eMost time on systema....:0  %s with %ld3a:0%02ld (wooow)\n",
                user[loop2].name, user[loop2].timeon/60/60, user[loop2].timeon/60 % 60);

        for (loop1=j, loop2=j; loop1<numusr; loop1++)
            if (!user [loop1].excl && (user[loop1].carrierd > user[loop2].carrierd))
                loop2=loop1;

        if (user [loop2].carrierd)
            fprintf (out, " 3eMost dropped carriersa..:0  %s with %ld of um.\n",
                user[loop2].name, user[loop2].carrierd);
      }
  }

/****************************************************************************/
/* parseargs...  deals with the command line options...                     */
/****************************************************************************/
void parseargs(int argc, char *argv[])
{
int i = 3;
char badarg=0;

  if ((argc > i) && (argv [i] [0] != '-') && (argv [i] [0] != '/'))
    {
      configfile = argv [i];
      i++;
    }

   readcfg ();

while(i<argc)
  {
  if ((argv [i] [0] != '-') && (argv [i] [0] != '/'))
    {
	  printf("unknown command line option \"%s\".\n", argv[i]);
      badarg = 1;
      break;
    }
  switch (toupper(argv[i][1]))
    {
    case '1':
	if(argv[i][2]=='+')
	  onetimecaller=1;
	else if(argv[i][2]=='-')
	  onetimecaller=0;
	break;

    case 'B':
	backdays = atoi (argv [i] + 2);
    if (backdays < 0) 
        backdays = 0;
	break;

    case 'G':
	if (argv [i] [2] == '+')
	  coloredbars=1;
	else if (argv [i] [2] == '-')
	  coloredbars=0;
	break;

    case 'C':
	if(argv[i][2]=='+')
	  color=1;
	else if(argv[i][2]=='-')
	  color=0;
	break;

    case 'D':
	debug=1;
	break;

    case 'U':
	if (argv [i] [2] == '+')
      {
        hourusage = 1;
        dayusage = 1;
      }
	else if (argv [i] [2] == '-')
      {
        hourusage = 0;
        dayusage = 0;
      }
    break;

    case '2':
	if (argv [i] [2] == '+')
	  twolineuser=1;
	else if (argv [i] [2] == '-')
	  twolineuser=0;
	break;

    case 'X':
	expdays = atoi (argv [i] + 2);
    if (expdays < 0) 
        expdays = 0;
	break;

    case '#':
	seeuser = atol (argv [i] + 2);
	break;

    case 'S':
	if(strlen(argv[i]) == 5)
	{	/* figure out the sort options */
    badarg = sortopt (&argv [i] [2]);
	}
	else
	{
	  printf("Wrong number of args in sort option. (must be \"-S123\".)\n");
	  badarg = 1;
	}
	break;

    default:
	printf("unknown command line option \"%s\".\n", argv[i]);
	badarg=1;
	break;
    }
  i++;
  }

/* ugly, isn't it?  yuck...oh well. */

if(badarg)
  {
  printf ("\n");
  show_usage (argv [0]);
  exit(1);
  }
}

/****************************************************************************/
/* save () -- saves the file, filtering all the ^A codes out if indicated.  */
/****************************************************************************/
void save (char *infile, char *outfile, int filter)
  {
    int ch=0, lch=0;

    in = fopen (infile, "rb");
    out = fopen (outfile, "wb");

    if (filter)
      {
        printf ("Filtering out color.\n");
        while (!feof (in))
          {
            lch = ch;
            ch = getc (in);

            if (ch == '' || lch == '')
                continue;
            putc(ch, out);
          }
      }
    else
      {
        while (!feof (in))
          {
            ch = getc (in);
            putc(ch, out);
          }
      }

    unlink (infile);
    fcloseall ();
  }

/****************************************************************************/
/* debug_out() -- makes a debug output file...  wow.    */
/****************************************************************************/
void debug_out(void)
{
   int loop1=0;

   if ((debugout = fopen("debug.out", "w")) == NULL)
   {
       printf("\n Can't open \"debug.out\" for output.\n");
       fcloseall();
       exit(-1);
   }

   fprintf(debugout,"5\n");
   fprintf(debugout,"version = %s\n\n", version);

   for (loop1=0; loop1<24; loop1++)
     fprintf(debugout,"call/hour # %d = %d\n", loop1, chour[loop1]);

   fprintf(debugout,"\n");

   for(loop1=0; loop1<7; loop1++)
     fprintf(debugout,"call/day # %d (%s) = %d\n",
	     loop1, DAYS[loop1], cday[loop1]);

   fprintf(debugout,"\n");

   for(loop1=0; loop1<7; loop1++)
     fprintf(debugout,"usedday # %d (%s) = %ld\n",
	     loop1, DAYS[loop1], usedday[loop1]);

   fprintf(debugout,"\n");

   for(loop1=0; loop1<24; loop1++)
     fprintf(debugout,"usedhour # %d = %ld\n", loop1, usedhour[loop1]);

   fprintf(debugout,"\n");

   for(loop1=0; loop1<16; loop1++)
     fprintf(debugout,"usedbaud # %d (%s) = %ld\n",
	     loop1, bauds[loop1], usedbaud[loop1]);

   fprintf(debugout,"\nnumusr = %d\n", numusr);
   fprintf(debugout,"numnet = %d\n", numnet);
   fprintf(debugout,"numproto = %d\n", numproto);
   fprintf(debugout,"fdate = %ld\n", fdate);
   fprintf(debugout,"ldate = %ld\n", ldate);
   fprintf(debugout,"downtime = %ld\n", downtime);
   fprintf(debugout,"networktime = %ld\n", networktime);
   fprintf(debugout,"usedtime = %ld\n", usedtime);
   fprintf(debugout,"logins = %ld\n", logins);
   fprintf(debugout,"hangups = %ld\n", hangups);
   fprintf(debugout,"chreq = %ld\n", chreq);
   fprintf(debugout,"menter = %ld\n", menter);
   fprintf(debugout,"mread = %ld\n", mread);
   fprintf(debugout,"anon = %ld\n", anon);
   fprintf(debugout,"newusr = %ld\n", newusr);
   fprintf(debugout,"wcup = %ld\n", wcup);
   fprintf(debugout,"wcdown = %ld\n", wcdown);
   fprintf(debugout,"net (net callin) = %ld\n", net);
   fprintf(debugout,"netcall (net callout) = %ld\n\n", netcall);

   for(loop1=0; loop1<numusr; loop1++)
     fprintf(debugout,"slt:%-2d nym:%-30s nw:%d tme:%-6ld cal:%-3ld cd:%-3ld up:%-3ld dwn:%-3ld R:%-5ld E:%-4ld anon:%-4ld cht:%-4ld\n",
	     loop1, user[loop1].name, user[loop1].new, user[loop1].timeon,
	     user[loop1].calls, user[loop1].carrierd, user[loop1].upload,
	     user[loop1].download, user[loop1].umread, user[loop1].umenter,
	     user[loop1].anon, user[loop1].uchats);

   fprintf(debugout,"\n");
   for(loop1=0; loop1<numnet; loop1++)
     fprintf(debugout,"slt:%-2d nym:%-30s tme:%-6ld cal:%-3ld bsy:%-3ld E:%-5ld R:%-5ld Dup:%-4ld exp:%-4ld\n",
	     loop1, network[loop1].name, network[loop1].timeon,
	     network[loop1].calls, network[loop1].busy,
	     network[loop1].nmenter, network[loop1].nmread,
	     network[loop1].nmduplicate, network[loop1].nmexpired);

   fprintf(debugout,"\n");
   for(loop1=0; loop1<numproto; loop1++)
     fprintf(debugout,"slt:%-2d nym:%-30s up:%ld dwn:%ld\n",
	     loop1, proto[loop1].name, proto[loop1].up, proto[loop1].down);
}
